# The command-line interface module creates an interface for
# interacting with the python program (wikicrawl). This is an implementation
# of the baker demo shown previously. The user can type in commands to
# make the program do things.

import sys
if sys.platform == 'linux':
    import readline # Needed for command history <up> and <down> arrows to work

from . import browser
from . import dal
from . import model

from app_skellington import _util
import app_skellington
from app_skellington._util import register_class_as_commands

# Problem pages:
# Decision (from politics)
# Malaysia (goes inside parenthesis)
# Soft-sediment_deformation_structures (doesn't find link)
# Chemicals (loops at philosophical)

class InteractiveInterface(app_skellington.ApplicationContainer):
    def __init__(self, *args, **kwargs):
        configspec_filepath = _util.get_asset(__name__, 'config.spec')
        config_filepath = self._get_config_filepath(
            'road2philosophy', '', 'config.ini'
        )

        super().__init__(
            configspec_filepath=configspec_filepath,
            config_filepath=config_filepath,
            *args, **kwargs
        )

        # Configure logging:
        # log_config = self.ctx.config['logging']
        # self.ctx.log.configure_logging(log_config)
        # self.ctx.log.configure_logging()

    def invoke_from_cli(self):
        rc = self.load_command()
        if not rc:
            print('Invalid command. Try -h for usage')
            return
        # load config
        self.invoke_command()

    def usage(self):
        s = '''
...usage info to come ;)...
'''
        print(s)

    def _cli_options(self):
        self.cli.add_argument(
            '--usage',
            help='Prints program usage information',
            action='store_true'
        )

    def _services(self):
        self['ctx'] = lambda: self.ctx

        self.dal = dal.DataLayer(self.ctx)
        self['dal'] = lambda: self.dal

        self.browser = browser.Browser(self.ctx)
        self['browser'] = lambda: self.browser

        self.model = model.Model(self.ctx, self.browser, self.dal)
        self['model'] = lambda: self.model

    def _command_menu(self):
        sm_root = self.cli.init_submenu('command')
        self.sm_root = sm_root

        register_class_as_commands(
            self, sm_root,
            model.Model
        )

def start_app(config=None):
    x = InteractiveInterface()
    x.invoke_from_cli()

